// A window putted into a class with WindowProcedure as a lambda

#include <Windows.h>
#include <memory>

class Window {
private:
	HWND MainWnd{};	
	WNDCLASSEX wndClass{};
public:
	bool Create();
	int MessageLoop();
};

bool Window::Create() {
	wndClass.cbSize = sizeof(WNDCLASSEX);
	wndClass.hInstance = GetModuleHandle(nullptr);
	wndClass.lpszClassName = L"Window";
	wndClass.lpfnWndProc = [](HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)->LRESULT
	{
		switch (Msg) {
		case WM_CREATE:
			return 0;

		case WM_CLOSE:
		case WM_DESTROY:
		case WM_QUIT:
			PostQuitMessage(0);

			return 0;
		}

		return DefWindowProc(hWnd, Msg, wParam, lParam);
	};

	if (!RegisterClassEx(&wndClass)) {
		return false;
	}

	MainWnd = CreateWindowEx(0, L"Window", L"Window", WS_OVERLAPPEDWINDOW, 0, 0, 800, 600, nullptr, nullptr, GetModuleHandle(nullptr), nullptr);

	if (!MainWnd) {
		return false;
	}

	ShowWindow(MainWnd, SW_NORMAL);
	UpdateWindow(MainWnd);

	return true;
}

int Window::MessageLoop() {
	MSG uMsg{};

	while (1) {
		if (PeekMessage(&uMsg, nullptr, 0, 0, PM_REMOVE)) {
			TranslateMessage(&uMsg);
			DispatchMessage(&uMsg);

			if (uMsg.message == WM_QUIT) break;
		}
	}

	return static_cast<int>(uMsg.wParam);
}

int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int) {
	auto Win = std::make_unique<Window>();

	Win->Create();

	return Win->MessageLoop();
}